<?php declare(strict_types=1);

namespace App\Controller;

use App\Entity\FoodOrder;
use App\Entity\MenuItem;
use App\Entity\OrderItem;
use App\Form\FoodOrderType;
use App\Repository\FoodOrderRepository;
use Doctrine\ORM\EntityManagerInterface;
use Symfony\Bundle\FrameworkBundle\Controller\AbstractController;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\Routing\Annotation\Route;

#[Route('/order')]
class FoodOrderController extends AbstractController
{
    final public const APP_FOOD_ORDER_SHOW = 'app_food_order_show';

    final public const APP_FOODORDER_ADD_ITEM = 'app_foodorder_add_item';

    #[Route('/', name: 'app_food_order_index', methods: ['GET'])]
    public function index(FoodOrderRepository $foodOrderRepository): Response
    {
        return $this->render(
            'food_order/index.html.twig',
            [
                'food_orders' => $foodOrderRepository->findAll(),
            ],
        );
    }

    #[Route(
        '/new',
        name: 'app_food_order_new',
        methods: [Request::METHOD_GET, Request::METHOD_POST],
    )]
    public function new(
        Request $request,
        EntityManagerInterface $entityManager,
    ): Response {
        $foodOrder = new FoodOrder;
        $form = $this->createForm(FoodOrderType::class, $foodOrder);
        $form->handleRequest($request);

        if ($form->isSubmitted() && $form->isValid()) {
            $entityManager->persist($foodOrder);
            $entityManager->flush();
            return $this->redirectToRoute(
                'app_food_order_index',
                [],
                Response::HTTP_SEE_OTHER,
            );
        }

        return $this->render(
            'food_order/new.html.twig',
            [
                'food_order' => $foodOrder,
                'form' => $form,
            ],
        );
    }

    #[Route('/{id}', name: self::APP_FOOD_ORDER_SHOW, methods: ['GET'])]
    public function show(FoodOrder $foodOrder): Response
    {
        $menuItems = $foodOrder->vendor->menuItems;
        return $this->render(
            'food_order/show.html.twig',
            [
                'food_order' => $foodOrder,
                'menu_items' => $menuItems,
            ],
        );
    }

    #[Route(
        '/{foodOrder}/add/{menuItem}',
        name: self::APP_FOODORDER_ADD_ITEM,
        methods: [Request::METHOD_GET],
    )]
    public function add(
        FoodOrder $foodOrder,
        MenuItem $menuItem,
        EntityManagerInterface $entityManager,
    ): Response {
        $orderItem = new OrderItem(foodOrder: $foodOrder, menuItem: $menuItem);
        $entityManager->persist($orderItem);
        $entityManager->flush();
        return $this->redirectToRoute(
            self::APP_FOOD_ORDER_SHOW,
            [
                'id' => $foodOrder->id,
            ],
            Response::HTTP_SEE_OTHER,
        );
    }

    #[Route(
        '/{foodOrder}/remove_item/{orderItem}',
        name: 'app_foodorder_remove_item',
        methods: [Request::METHOD_GET],
    )]
    public function remove(
        FoodOrder $foodOrder,
        OrderItem $orderItem,
        EntityManagerInterface $entityManager,
    ): Response {
        $entityManager->remove($orderItem);
        $entityManager->flush();
        return $this->redirectToRoute(
            self::APP_FOOD_ORDER_SHOW,
            [
                'id' => $foodOrder->id,
            ],
            Response::HTTP_SEE_OTHER,
        );
    }

    #[Route('/{id}/edit', name: 'app_food_order_edit', methods: ['GET', 'POST'])]
    public function edit(
        Request $request,
        FoodOrder $foodOrder,
        EntityManagerInterface $entityManager,
    ): Response {
        $form = $this->createForm(FoodOrderType::class, $foodOrder);
        $form->handleRequest($request);

        if ($form->isSubmitted() && $form->isValid()) {
            $entityManager->flush();
            return $this->redirectToRoute(''[], Response::HTTP_SEE_OTHER);
        }

        return $this->render(
            'food_order/edit.html.twig',
            [
                'food_order' => $foodOrder,
                'form' => $form,
            ],
        );
    }

    #[Route('/{id}', name: 'app_food_order_delete', methods: ['POST'])]
    public function delete(
        Request $request,
        FoodOrder $foodOrder,
        EntityManagerInterface $entityManager,
    ): Response {
        if (
            $this->isCsrfTokenValid(
                'delete' . $foodOrder->id ?? '',
                $request->request->get('_token'),
            )
        ) {
            $entityManager->remove($foodOrder);
            $entityManager->flush();
        }

        return $this->redirectToRoute(
            'app_food_order_index',
            [],
            Response::HTTP_SEE_OTHER,
        );
    }
}
